Package: hutilscpp
Title: Miscellaneous Functions in C++
Version: 0.10.10
Authors@R: 
    c(person(given = "Hugh",
           family = "Parsonage",
           role = c("aut", "cre"),
           email = "hugh.parsonage@gmail.com"),
      person(given = "Simon", 
              family = "Urbanek",
              role = "ctb",
              comment = "fastmatch components"))
Description: Provides utility functions that are simply, frequently used, 
    but may require higher performance that what can be obtained from base R.
    Incidentally provides support for 'reverse geocoding', such as matching a point
    with its nearest neighbour in another array. Used as a complement to package
    'hutils' by sacrificing compilation or installation time for higher running 
    speeds. The name is a portmanteau of the author and 'Rcpp'.
URL: https://github.com/hughparsonage/hutilscpp
BugReports: https://github.com/hughparsonage/hutilscpp/issues
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.3.0)
Imports: data.table, hutils, magrittr, utils
RoxygenNote: 7.3.2
Suggests: bench, parallel, TeXCheckR, withr, tinytest, covr
NeedsCompilation: yes
Packaged: 2025-03-20 07:45:30 UTC; hughp
Author: Hugh Parsonage [aut, cre],
  Simon Urbanek [ctb] (fastmatch components)
Maintainer: Hugh Parsonage <hugh.parsonage@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-20 08:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 19:22:08 UTC; unix
Archs: hutilscpp.so.dSYM
