# Generated by roxygen2: do not edit by hand

export(COALESCE0)
export(Comma)
export(Implies)
export(ModeC)
export(abs_diff)
export(allNA)
export(and3)
export(and3s)
export(anyOutside)
export(are_even)
export(as_integer_if_safe)
export(bench_system_time)
export(character2integer)
export(coalesce0)
export(count_logical)
export(cumsum_reset)
export(diam)
export(divisible)
export(divisible16)
export(divisible2)
export(every_int)
export(finp)
export(fmatchp)
export(fnotinp)
export(helper)
export(is_constant)
export(is_sorted)
export(isntConstant)
export(isntSorted)
export(match_nrst_haversine)
export(max_abs_diff)
export(minmax)
export(or3)
export(or3s)
export(pmax0)
export(pmax3)
export(pmaxC)
export(pmaxV)
export(pmin0)
export(pmin3)
export(pminC)
export(pminV)
export(poleInaccessibility2)
export(poleInaccessibility3)
export(range_rcpp)
export(squish)
export(sum_and3s)
export(sum_isna)
export(sum_or3s)
export(thinner)
export(uniqueN_fmatch)
export(unique_fmatch)
export(which3)
export(which_are_even)
export(which_first)
export(which_firstNA)
export(which_last)
export(which_lastNA)
export(which_true_onwards)
export(whichs)
export(xor2)
import(data.table)
importFrom(hutils,"%<->%")
importFrom(hutils,AND)
importFrom(hutils,NOR)
importFrom(hutils,XOR)
importFrom(magrittr,and)
importFrom(magrittr,or)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,ls.str)
useDynLib(hutilscpp, .registration=TRUE)
